#pragma once
#include <SDL_events.h>
#include <unordered_map>
#include <vector>

#include "GamePad.h"
#include "KActionBind.h"

namespace KapitanGame {
	class KInput {
		std::vector<SDL_GameController*> ConnectedControllers;
		std::vector<GamePad> ControllerInputs;
		std::vector<GamePad> LastControllerInputs;
		std::vector<Uint8> KeyboardInputs;
		std::vector<Uint8> LastKeyboardInputs;
		std::vector<KActionBind> Actions;
		std::vector<KAxisBind> Axes;
		std::unordered_map<std::string, Uint8> KeyBinds[static_cast<int>(Controllers::Controller4) + 1];
		std::unordered_map<std::string, SDL_GameControllerButton> ButtonBinds[static_cast<int>(Controllers::Controller4) + 1];
		int GamePadsCount;
		bool Initialized;

		bool IsControllerButtonPressed(Controllers controllerId, SDL_GameControllerButton button) const;
		bool IsControllerButtonReleased(Controllers controllerId, SDL_GameControllerButton button) const;
		bool IsControllerButtonHeld(Controllers controllerId, SDL_GameControllerButton button) const;
		float GetControllerAxis(Controllers controllerId, SDL_GameControllerAxis axis) const;
		bool IsKeyboardButtonHeld(Uint8 scanCode) const;
		bool IsKeyboardButtonPressed(Uint8 scanCode) const;
		bool IsKeyboardButtonReleased(Uint8 scanCode) const;
	public:
		KInput();
		void Init();
		void Free();
		void HandleInputPreEvents();
		void HandleInputPostEvents();
		void HandleEvent(const SDL_Event& event);
		void BindAction(const std::string& name, InputState expectedInputState, KPlayerController* controllerObject, KPlayerCommand command, Controllers
			controller);
		void BindAxis(const std::string& name, KPlayerController* controllerObject, KPlayerAxisCommand command, Controllers controller);
		bool CheckAction(const std::string& name, InputState expectedInputState, Controllers controller);
		void ProcessActions();
	};
}


